/* drotg.f -- translated by f2c (version 19991025). You must link the resulting object file with the libraries: -lf2c -lm (in that order) */
#include "FLA_f2c.h"
/* Table of constant values */
static doublereal c_b4 = 1.;
/* Subroutine */
int drotg_(doublereal *da, doublereal *db, doublereal *c__, doublereal *s)
{
    /* System generated locals */
    doublereal d__1, d__2;
    /* Builtin functions */
    double sqrt(doublereal), d_sign(doublereal *, doublereal *);
    /* Local variables */
    doublereal r__, scale, z__, roe;
    /* construct givens plane rotation. */
    /* jack dongarra, linpack, 3/11/78. */
    roe = *db;
    if (f2c_abs(*da) > f2c_abs(*db))
    {
        roe = *da;
    }
    scale = f2c_abs(*da) + f2c_abs(*db);
    if (scale != 0.)
    {
        goto L10;
    }
    *c__ = 1.;
    *s = 0.;
    r__ = 0.;
    z__ = 0.;
    goto L20;
L10: /* Computing 2nd power */
    d__1 = *da / scale;
    /* Computing 2nd power */
    d__2 = *db / scale;
    r__ = scale * sqrt(d__1 * d__1 + d__2 * d__2);
    r__ = d_sign(&c_b4, &roe) * r__;
    *c__ = *da / r__;
    *s = *db / r__;
    z__ = 1.;
    if (f2c_abs(*da) > f2c_abs(*db))
    {
        z__ = *s;
    }
    if (f2c_abs(*db) >= f2c_abs(*da) && *c__ != 0.)
    {
        z__ = 1. / *c__;
    }
L20:
    *da = r__;
    *db = z__;
    return 0;
}
/* drotg_ */

