/*

    Copyright (C) 2014, The University of Texas at Austin

    This file is part of libflame and is available under the 3-Clause
    BSD license, which can be found in the LICENSE file at the top-level
    directory, or at http://opensource.org/licenses/BSD-3-Clause

*/
FLA_Error REF_Chol( FLA_Obj );

FLA_Error Chol_unb_var1( FLA_Obj );
FLA_Error Chol_unb_var2( FLA_Obj );
FLA_Error Chol_unb_var3( FLA_Obj );

FLA_Error Chol_blk_var1( FLA_Obj, int );
FLA_Error Chol_blk_var2( FLA_Obj, int );
FLA_Error Chol_blk_var3( FLA_Obj, int );
