/* dswap.f -- translated by f2c (version 19991025). You must link the resulting object file with the libraries: -lf2c -lm (in that order) */
#include "FLA_f2c.h"
/* Subroutine */
int dswap_(integer *n, doublereal *dx, integer *incx, doublereal *dy, integer *incy)
{
    /* System generated locals */
    integer i__1;
    /* Local variables */
    integer i__, m;
    doublereal dtemp;
    integer ix, iy, mp1;
    /* interchanges two vectors. */
    /* uses unrolled loops for increments equal one. */
    /* jack dongarra, linpack, 3/11/78. */
    /* modified 12/3/93, array(1) declarations changed to array(*) */
    /* Parameter adjustments */
    --dy;
    --dx;
    /* Function Body */
    if (*n <= 0)
    {
        return 0;
    }
    if (*incx == 1 && *incy == 1)
    {
        goto L20;
    }
    /* code for unequal increments or equal increments not equal */
    /* to 1 */
    ix = 1;
    iy = 1;
    if (*incx < 0)
    {
        ix = (-(*n) + 1) * *incx + 1;
    }
    if (*incy < 0)
    {
        iy = (-(*n) + 1) * *incy + 1;
    }
    i__1 = *n;
    for (i__ = 1;
            i__ <= i__1;
            ++i__)
    {
        dtemp = dx[ix];
        dx[ix] = dy[iy];
        dy[iy] = dtemp;
        ix += *incx;
        iy += *incy;
        /* L10: */
    }
    return 0;
    /* code for both increments equal to 1 */
    /* clean-up loop */
L20:
    m = *n % 3;
    if (m == 0)
    {
        goto L40;
    }
    i__1 = m;
    for (i__ = 1;
            i__ <= i__1;
            ++i__)
    {
        dtemp = dx[i__];
        dx[i__] = dy[i__];
        dy[i__] = dtemp;
        /* L30: */
    }
    if (*n < 3)
    {
        return 0;
    }
L40:
    mp1 = m + 1;
    i__1 = *n;
    for (i__ = mp1;
            i__ <= i__1;
            i__ += 3)
    {
        dtemp = dx[i__];
        dx[i__] = dy[i__];
        dy[i__] = dtemp;
        dtemp = dx[i__ + 1];
        dx[i__ + 1] = dy[i__ + 1];
        dy[i__ + 1] = dtemp;
        dtemp = dx[i__ + 2];
        dx[i__ + 2] = dy[i__ + 2];
        dy[i__ + 2] = dtemp;
        /* L50: */
    }
    return 0;
}
/* dswap_ */

