#!/bin/bash

cp FLA_Fused_Ahx_Ax_opt_var1.c.orig        FLA_Fused_Ahx_Ax_opt_var1.c
cp FLA_Fused_Uhu_Yhu_Zhu_opt_var1.c.orig   FLA_Fused_Uhu_Yhu_Zhu_opt_var1.c
cp FLA_Fused_Gerc2_Ahx_Ax_opt_var1.c.orig  FLA_Fused_Gerc2_Ahx_Ax_opt_var1.c
