#!/bin/bash

cp FLA_Fused_Ahx_Axpy_Ax_opt_var1.c.orig       FLA_Fused_Ahx_Axpy_Ax_opt_var1.c
cp FLA_Fused_Gerc2_Ahx_Axpy_Ax_opt_var1.c.orig FLA_Fused_Gerc2_Ahx_Axpy_Ax_opt_var1.c
cp FLA_Fused_Gerc2_opt_var1.c.orig             FLA_Fused_Gerc2_opt_var1.c
cp FLA_Fused_UYx_ZVx_opt_var1.c.orig           FLA_Fused_UYx_ZVx_opt_var1.c

